/*
 * Game.cpp
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "Game.h"
#include "BSPTree.h"
#include "Level.h"
#include "Triangle.h"
#include "gl.h"

namespace Zoom {

Game::Game(bool multisample) : angle(0) {
  glClearColor(0.0, 0.0, 0.0, 1.0);
  glClearDepth(1.0);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);

  //glEnable(GL_BLEND);
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

#ifndef _WIN32
  if(multisample)
    glEnable(GL_MULTISAMPLE_ARB);
#endif

  glShadeModel(GL_SMOOTH);

  glEnable(GL_LIGHTING);
  static const float light[] = {0, 0, 0, 1};
  static const float lightColor[] = {1, 1, 1, 1};
  glLightfv(GL_LIGHT0, GL_POSITION, light);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
  glEnable(GL_LIGHT0);

  glEnable(GL_COLOR_MATERIAL);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);

  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CCW);

  glLoadIdentity();

  loadLevel("level.xml");
}

bool Game::loadLevel(const std::string &name) {
  level = Level::loadLevel(name);

  return level;
}

void Game::run(int delta) {
  angle += delta*0.01;

  if(angle >= 360)
    angle -= 360;
}

void Game::render() {
  std::list<BSPTree::TriangleRecord> triangles;

  triangles.insert(triangles.end(), level->getRooms().front().walls.begin(), level->getRooms().front().walls.end());

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glPushMatrix();
  glRotatef(angle, 1, 2, 1);
  //glRotatef(3*angle, 0, -1, 2);
  //glRotatef(5*angle, 2, -1, 0);

  renderer.render(triangles);

  glPopMatrix();
}

}
