/*
 * Renderer.h
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZOOM_RENDERER_H_
#define ZOOM_RENDERER_H_

#include "BSPTree.h"

namespace Zoom {

class Renderer {
  public:
    Renderer() : lastTexture(0), renderVisitor(this) {}

    void render(const BSPTree &tree);
    void render(const std::list<BSPTree::TriangleRecord> &triangles);

  private:
    void renderTriangle(const Triangle &t);

    class RenderVisitor {
      public:
        RenderVisitor(Renderer *renderer0) : renderer(renderer0) {}

        void operator() (const BSPTree::TriangleRecord &t) const {
          renderer->renderTriangle(t.triangle);
        }

      private:
        Renderer *renderer;
    };

    unsigned lastTexture;

    const RenderVisitor renderVisitor;
};

}

#endif /* ZOOM_RENDERER_H_ */
